
	list	p=16F876
	#include P16F876.inc
	#include capa.inc
	#include macro.inc

	errorlevel -302

	extern	counter, delay, temp_wr, temp_rd
	extern	delay_100us, delay_4ms, delay_15ms


lcd_code	CODE

	
LCDLine_1
	banksel	temp_wr
	movlw	0x80
	movwf	temp_wr
	call	i_write
	return
	GLOBAL	LCDLine_1

LCDLine_2
	banksel	temp_wr
	movlw	0xC0
	movwf	temp_wr
	call	i_write
	return
	GLOBAL	LCDLine_2

LCDClear
	banksel	temp_wr
	movlw	b'00000001'		;#7   Display Clear
	movwf	temp_wr
	call	i_write
	return
	GLOBAL	LCDClear

d_write					;write data
	call	LCDBusy
	bsf	STATUS, C	
	call	LCDWrite
	return
	GLOBAL	d_write

i_write					;write instruction
	call	LCDBusy
	bcf	STATUS, C
	call	LCDWrite
	return
 	GLOBAL	i_write

rlcd	macro	MYREGISTER
 IF MYREGISTER == 1
	bsf	STATUS, C
	call	LCDRead
 ELSE
	bcf	STATUS, C
	call	LCDRead
 ENDIF
	endm
;****************************************************************************




; *******************************************************************
LCDInit
	clrf	PORTB
	
	banksel	TRISB			;configure control lines
	bcf	LCD_E_DIR
	bcf	LCD_RW_DIR
	bcf	LCD_RS_DIR
	
	call	delay_15ms
	
	movlw	b'00110000'		;#1 Send control sequence 
	movwf	temp_wr
	bcf	STATUS,C
	call	LCDWriteNibble

	call	delay_4ms

	movlw	b'00110000'		;#2 Send control sequence
	movwf	temp_wr
	bcf	STATUS,C
	call	LCDWriteNibble

	call	delay_100us
						
	movlw	b'0011000'		;#3 Send control sequence
	movwf	temp_wr
	bcf	STATUS,C
	call	LCDWriteNibble

	call	delay_100us

	movlw	b'00100000'		;#4 set 4-bit
	movwf	temp_wr
	bcf	STATUS,C
	call	LCDWriteNibble

	call	LCDBusy			;Busy?
				
	movlw	b'00101000'		;#5   Function set (0010NFxx) N=1 2 lines
	movwf	temp_wr			;     F=0 5x7 font
	call	i_write

	movlw	b'00001100'		;#6  Display = ON  0000.1DCB
	movwf	temp_wr			; D:display on   C=custor on  B=blink
	call	i_write
			
	movlw	b'00000001'		;#7   Display Clear
	movwf	temp_wr
	call	i_write

	movlw	b'00000110'		;#8   Entry Mode  0000.01IS
	movwf	temp_wr			; I=1:increment cursor I=0:decrement
	call	i_write			; S=1:display shift  S=0:no shift

	movlw	b'10000000'		;DDRAM addresss 0000
	movwf	temp_wr
	call	i_write

	;movlw	b'00000010'		;return home
	;movwf	temp_wr
	;call	i_write


	return

	GLOBAL	LCDInit	
; *******************************************************************








;****************************************************************************
;     _    ______________________________
; RS  _>--<______________________________
;     _____
; RW       \_____________________________
;                  __________________
; E   ____________/                  \___
;     _____________                ______
; DB  _____________>--------------<______
;
LCDWriteNibble
	btfss	STATUS, C		; Set the register select
	bcf	LCD_RS
	btfsc	STATUS, C	
	bsf	LCD_RS

	bcf	LCD_RW			; Set write mode

	banksel	TRISB
	bcf	LCD_D4_DIR		; Set data bits to outputs
	bcf	LCD_D5_DIR
	bcf	LCD_D6_DIR
	bcf	LCD_D7_DIR

	NOP				; Small delay
	NOP

	banksel	PORTB
	bsf	LCD_E			; Setup to clock data
	
	btfss	temp_wr, 7			; Set high nibble
	bcf	LCD_D7	
	btfsc	temp_wr, 7
	bsf	LCD_D7
	btfss	temp_wr, 6
	bcf	LCD_D6	
	btfsc	temp_wr, 6
	bsf	LCD_D6
	btfss	temp_wr, 5
	bcf	LCD_D5	
	btfsc	temp_wr, 5
	bsf	LCD_D5
	btfss	temp_wr, 4
	bcf	LCD_D4
	btfsc	temp_wr, 4
	bsf	LCD_D4	

	NOP
	NOP

	bcf	LCD_E			; Send the data

	return
; *******************************************************************





; *******************************************************************
LCDWrite
	call	LCDWriteNibble
	BANKSEL	temp_wr
	swapf	temp_wr, f
	call	LCDWriteNibble
	banksel	temp_wr
	swapf	temp_wr,f

	return

	GLOBAL	LCDWrite
; *******************************************************************





; *******************************************************************
;     _____    _____________________________________________________
; RS  _____>--<_____________________________________________________
;               ____________________________________________________
; RW  _________/
;                  ____________________      ____________________
; E   ____________/                    \____/                    \__
;     _________________                __________                ___
; DB  _________________>--------------<__________>--------------<___
;
LCDRead
	banksel	TRISB
	bsf	LCD_D4_DIR		; Set data bits to inputs
	bsf	LCD_D5_DIR
	bsf	LCD_D6_DIR
	bsf	LCD_D7_DIR		

	BANKSEL	PORTB
	btfss	STATUS, C		; Set the register select
	bcf	LCD_RS
	btfsc	STATUS, C	
	bsf	LCD_RS

	bsf	LCD_RW			;Read = 1

	NOP
	NOP			

	bsf	LCD_E			; Setup to clock data

	NOP
	NOP
	NOP
	NOP

	btfss	LCD_D7			; Get high nibble
	bcf	temp_rd, 7
	btfsc	LCD_D7
	bsf	temp_rd, 7
	btfss	LCD_D6			
	bcf	temp_rd, 6
	btfsc	LCD_D6
	bsf	temp_rd, 6
	btfss	LCD_D5			
	bcf	temp_rd, 5
	btfsc	LCD_D5
	bsf	temp_rd, 5
	btfss	LCD_D4			
	bcf	temp_rd, 4
	btfsc	LCD_D4
	bsf	temp_rd, 4

	bcf	LCD_E			; Finished reading the data

	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	bsf	LCD_E			; Setup to clock data

	NOP
	NOP

	btfss	LCD_D7			; Get low nibble
	bcf	temp_rd, 3
	btfsc	LCD_D7
	bsf	temp_rd, 3
	btfss	LCD_D6			
	bcf	temp_rd, 2
	btfsc	LCD_D6
	bsf	temp_rd, 2
	btfss	LCD_D5			
	bcf	temp_rd, 1
	btfsc	LCD_D5
	bsf	temp_rd, 1
	btfss	LCD_D4			
	bcf	temp_rd, 0
	btfsc	LCD_D4
	bsf	temp_rd, 0

	bcf	LCD_E			; Finished reading the data

FinRd
	return
; *******************************************************************






; *******************************************************************
LCDBusy
					; Check BF
	rlcd	LCD_INS
	btfsc	temp_rd, 7
	goto	LCDBusy
	return

	GLOBAL	LCDBusy
; *******************************************************************





	END
