;-----------------------------------------------------------

BANK0	MACRO
		bcf		STATUS, RP0
		bcf		STATUS, RP1
		bcf		STATUS, IRP
		ENDM

BANK1	MACRO
		bsf		STATUS, RP0
		bcf		STATUS, RP1
		bcf		STATUS, IRP
		ENDM

BANK2	MACRO
		bcf		STATUS, RP0
		bsf		STATUS, RP1
		bsf		STATUS, IRP
		ENDM

BANK3	MACRO
		bsf		STATUS, RP0
		bsf		STATUS, RP1
		bsf		STATUS, IRP
		ENDM

PAGE0	MACRO
		bcf		PCLATH, 3
		bcf		PCLATH, 4
		ENDM

PAGE1	MACRO
		bsf		PCLATH, 3
		bcf		PCLATH, 4
		ENDM

PAGE2	MACRO
		bcf		PCLATH, 3
		bsf		PCLATH, 4
		ENDM

PAGE3	MACRO
		bsf		PCLATH, 3
		bsf		PCLATH, 4
		ENDM


; TABLE_JUMP  Calculates an eventual page boundary crossing
; - set up the PCLATH register correctly
; - offset must be in W, offset 0 jumps to the next instr.
; - uses one byte of dedicated ram

TABLE_JUMP	MACRO
		movwf	Table_Temp		; save wanted offset
		movlw	LOW($+8)		; get low address
		addwf	Table_Temp,f		; add offset
		movlw	HIGH($+6)		; get highest 5 bits
		btfsc	STATUS, C		; page crossed ?
		addlw	0x01			; yes, add 1 to high address
		movwf	PCLATH			; load high address in latch
		movf	Table_Temp,w		; get computed address
		movwf	PCL			; and jump.
		ENDM


; PUSH/PULL : save and restore W, PCLATH, STATUS and FSR registers

PUSH	MACRO
		movwf	Saved_W			; save W
		swapf	STATUS,w		; swapf doesn't affect status bits
		clrf	STATUS			; bank 0
		movwf	Saved_Status	; save STATUS
		movf	PCLATH,w
		movwf	Saved_Pclath	; save PCLATH
		clrf	PCLATH
		movf	FSR,w
		movwf	Saved_Fsr		; save FSR
		ENDM

PULL	MACRO
		movf	Saved_Fsr,w
		movwf	FSR				; restore FSR
		movf	Saved_Pclath,w
		movwf	PCLATH			; restore PCLATH
		swapf	Saved_Status,w
		movwf	STATUS			; restore STATUS (and bank)
		swapf	Saved_W,f		; reload into self to set status bits
		swapf	Saved_W,w		; and restore
		ENDM


; DISABLE_IRQ  disable interrupts

DISABLE_IRQ	MACRO
	LOCAL	Stop_int
Stop_int
		bcf	INTCON,GIE
		btfsc	INTCON,GIE
		goto	Stop_int
		ENDM

; ENABLE_IEQ  enable interrupts

ENABLE_IRQ	MACRO
		bsf		INTCON,GIE
		ENDM



; INC_BUFFER : advance in buffer, handle page boundaries properly.
;
; if buffer size is a power of 2 and buffer is aligned on a multiple
; of twice its size, this macro generates 2 instructions, and 6
; otherwise.

INC_BUFFER	MACRO	Pointer, Base, Size
	LOCAL	Power_of_2, Aligned, Bit, Value

Power_of_2	SET		!(Size&(Size-1))
Aligned		SET		Power_of_2

	IF	Aligned

Value		SET		Size
Bit			SET		0

	WHILE	Value>1
Bit			SET		Bit+1
Value		SET		Value>>1
	ENDW

	ENDIF

	incf	Pointer,f			; increment pointer

	IF	Aligned&&!(Base&(1<<Bit))

	bcf		Pointer,Bit

	ELSE

	movf	Pointer,w
	xorlw	Base+Size
	movlw	Base
	btfsc	STATUS,Z
	movwf	Pointer

	ENDIF

	ENDM



; convert W to upper case
TO_UPPER	MACRO
	addlw	.133
	addlw	.26
	btfss	STATUS,C
	addlw	0x20
	addlw	.65
	ENDM
