	list p=16f876

	include p16f876.inc
	include capa.inc
	include macro.inc

	errorlevel -302

	__CONFIG _CP_OFF & _WDT_OFF & _HS_OSC & _LVP_OFF & _BODEN_ON


bank0_data	UDATA 0x20

temp_1		res 	1
temp_2		res 	1
temp_3 		res 	1
temp_4 		res 	1

counter		res	1
delay		res	1
temp_wr		res	1
temp_rd		res	1
ptr_count	res	1
ptr_pos		res	1

capHH 		res 	1
capH 		res 	1
capL 		res 	1

cap2HH 		res 	1
cap2H 		res 	1
cap2L 		res 	1

type 		res 	1
count		res	1

zeroHH 		res 	1
zeroH 		res 	1
zeroL 		res 	1
	
zeroESR_H 	res 	1
zeroESR_L 	res 	1
zeroAC_H	res	1
zeroAC_L	res	1
zeroDC_H	res	1
zeroDC_L	res	1

cap1H 		res 	1
cap1L 		res 	1

TenK 		res 	1	
Thou 		res 	1
Hund 		res 	1
Tens 		res 	1
Ones 		res 	1	

divH 		res 	1
divL 		res 	1



bank0123_data	UDATA_SHR 0x70

ICDReserved_1 	res 	1
Saved_W 	res 	1
Saved_Status 	res 	1
Saved_Pclath 	res 	1
Saved_Fsr 	res 	1
Table_Temp 	res 	1
Flag1 		res 	1
address		res	1
value		res	1



	GLOBAL	temp_1, temp_2, temp_3, temp_4
	GLOBAL	counter, delay, temp_wr, temp_rd

	extern	delay_100us, delay_1ms, delay_4ms, delay_10ms, delay_15ms
	extern	delay_100ms, delay_1s, delay_2s, delay_10s, delay_1min

	extern	LCDInit, LCDClear, LCDLine_1, LCDLine_2, d_write, i_write



; boot vector
	ORG 0x000
	nop

	goto	start


; interrupt code
	ORG 0x0004
	goto	int_handler



; *********************
; * INTERRUPT HANDLER *
; *********************
int_code	CODE

int_handler
	PUSH			; save registers
int_test_timer1
	btfss	PIR1,TMR1IF
	goto	int_test_ccp2
	call	timer1_handler
	bcf	PIR1,TMR1IF
int_test_ccp2
	btfss	PIR2,CCP2IF
	goto	int_test_xx
	call	ccp2_handler
	bcf	PIR2,CCP2IF
int_test_xx
	
int_exit
	PULL			; restore registers
	retfie

;------------------ CCP2 handler --------------------------------
ccp2_handler
	bsf	captured
	return

;------------------ TIMER1 handler ------------------------------
timer1_handler
	btfsc	doing_mF
	goto	timer1_mF
	bsf	overflow
	incf	capHH,f
	return
timer1_mF
	incf	capHH,f
	btfsc	STATUS,C
	bsf	overflow
	return



; **********
; * EEPROM *
; **********
eeprom_1	CODE 0x2100
	de	.0, .8, .0, .7


; *************
; * MAIN CODE *
; *************
main_code	CODE



start
	call	clear_ram
	call	init_ports
	call	LCDInit

	ENABLE_IRQ
	
	BANK0

	call	stan_char_1		; ptr_pos = 0, so this display the
	call	stan_char_2		; welcome message
	call	delay_1s

	bsf	RESET555
	bsf	TRIG555

;	btfss	CALI_PIN
;	call	calibration

	call	battery_check

	call	zero_pF
	call	zero_ESR
	call	find_mode
	btfsc	STATUS,C
	goto	ESR_mode
	goto	capacitance_mode



; *****************
; * Battery check *
; *****************
; battery is 9V, going to a voltage divider where it is divided by 2.
; when battery is 6V, we want to issue a "low battery" warning. 6V will
; be 3V at the input of the A/D converter. max scale is 5V, so 3V will be
; 3/5 of the value given after A/D. We have 10 bits resolution, but we 
; will drop the 2 LSB. 3/5*1024 / 4 = 153
battery_check
	call	read_AD0
	rrf	capH,f		; drop 2 MSB
	rrf	capL,f
	rrf	capH,f
	rrf	capL,f
	movf	capL,w		; capL is our value
	sublw	.153
	btfss	STATUS,C
	return
	call	LCDClear
	movlw	.32
	movwf	ptr_pos
	call	stan_char_1
	call	stan_char_2
	call	delay_2s
	return


; *******************
; * ESR calibration *
; *******************
calibration
	movlw	.88
	movwf	ptr_pos
	call	stan_char_1
	call	stan_char_2

	; AC
	movlw	B'00001111'	; 2 LSB='00', PWM mode
	movwf	CCP1CON
	bsf	T2CON,TMR2ON	; start timer2
	call	delay_1s
	call	read_AD1
	movf	capH,w
	movwf	zeroAC_H
	movf	capL,w
	movwf	zeroAC_L
	
	; DC
	bcf	T2CON,TMR2ON	; stop timer2
	clrf	CCP1CON		; reset CCP1 module
	bsf	PORTC,2		; output high
	call	delay_100ms
	call	read_AD1
	movf	capH,w
	movwf	zeroDC_H
	movf	capL,w
	movwf	zeroDC_L

	; save in eeprom
	call	save_zero_ESR

	call	LCDClear
	movlw	.104
	call	stan_char_1
	call	delay_1s

	return


; save calibration values into eeprom
save_zero_ESR
	clrf	address
	movf	zeroAC_H,w
	call	eeprom_write_byte
	incf	address,f
	movf	zeroAC_L,w
	call	eeprom_write_byte
	incf	address,f
	movf	zeroDC_H,w
	call	eeprom_write_byte
	incf	address,f
	movf	zeroDC_L,w
	call	eeprom_write_byte
	return

; restore zero values from previous calibration
zero_ESR
	clrf	address
	call	eeprom_read_byte
	movwf	zeroAC_H
	incf	address,f
	call	eeprom_read_byte
	movwf	zeroAC_L
	incf	address,f
	call	eeprom_read_byte
	movwf	zeroDC_H
	incf	address,f
	call	eeprom_read_byte
	movwf	zeroDC_L
	return



; *******
; * LCD *
; *******
stan_table
	TABLE_JUMP
	;  "01234567"
	dt "Capacime"	; 0
	dt "ter  1.0"   ; 8
	dt " Capa : "	; 16
	dt "  ESR : "   ; 24
	dt " Low Bat"	; 32
	dt "tery !  "	; 40
	dt " Error !"	; 48
	dt "Range Ex"	; 56
	dt "ceeded !"	; 64
	dt "Remove C"	; 72
	dt "ap first"	; 80
	dt "Calibrat"	; 88
	dt "ion mode"	; 96
	dt "Done !"	; 104

stan_char_1
	call	LCDLine_1
	banksel	ptr_count
	movlw	.8
	movwf	ptr_count
stan_next_char_1
	movlw	HIGH stan_table
	movwf	PCLATH
	movf	ptr_pos, w
	call	stan_table
	movwf	temp_wr
	call	d_write
	banksel	ptr_pos
	incf	ptr_pos, f
	decfsz	ptr_count, f
	goto	stan_next_char_1
	return


stan_char_2
	call	LCDLine_2
	banksel	ptr_count
	movlw	.8
	movwf	ptr_count
stan_next_char_2
	movlw	HIGH stan_table
	movwf	PCLATH
	movf	ptr_pos, w
	call	stan_table
	movwf	temp_wr
	call	d_write
	banksel	ptr_pos
	incf	ptr_pos, f
	decfsz	ptr_count, f
	goto	stan_next_char_2
	return

a_write
	addlw	'0'
	movwf	temp_wr
	goto	d_write




; -------------------- find mode ------------------------------------
; find in which mode we are : ESR or capacitance. We cheat by looking
; at the "-" pin of the test capacitor. In capacitance mode, this pin
; is tied to ground, so we read 0. In ESR mode, we should read a value
; > 0.
; RETURN: carry set if ESR mode
;         carre unset if capacitance mode
find_mode
	call	read_AD2
	bsf	STATUS,C
	movf	capH,f
	btfss	STATUS,Z
	return
	movlw	.50		; keep some marging for > 0 test
	subwf	capL,w
	return


; --------------------- ESR mode -------------------------------
ESR_mode
	call	LCDClear
	bsf	first
ESR_mode_loop

ESR_mode_AC
	movlw	B'00001111'	; 2 LSB='00', PWM mode
	movwf	CCP1CON
	bsf	T2CON,TMR2ON	; start timer2
	clrf	type
	call	delay_100ms
	movf	zeroAC_H,w
	movwf	zeroESR_H
	movf	zeroAC_L,w
	movwf	zeroESR_L
	call	ESR_read

	call	find_mode
	btfss	STATUS,C
	goto	capacitance_mode

ESR_mode_DC
	bcf	T2CON,TMR2ON	; stop timer2
	clrf	CCP1CON		; reset CCP1 module
	bsf	PORTC,2		; output high
	movlw	.1
	movwf	type
	call	delay_100ms
	movf	zeroDC_H,w
	movwf	zeroESR_H
	movf	zeroDC_L,w
	movwf	zeroESR_L
	call	ESR_read

	call	find_mode
	btfss	STATUS,C
	goto	capacitance_mode

	call	delay_100ms

	goto	ESR_mode_loop


ESR_read
	call	delay_100ms
	call	read_AD1
	movf	zeroESR_L,w
	subwf	capL,f
	movf	zeroESR_H,w
	btfss	STATUS,C
	incfsz	zeroESR_H,w
	subwf	capH,f
	btfsc	STATUS,C
	goto	ESR_display
	clrf	capH
	clrf	capL

ESR_display
	movf	type,f
	btfss	STATUS,Z
	goto	ESR_display_DC
	call	LCDLine_1
	movlw	'A'-'0'
	call	a_write
	goto	ESR_display_1
ESR_display_DC
	call	LCDLine_2
	movlw	'D'-'0'
	call	a_write
ESR_display_1
	movlw	'C'-'0'
	call	a_write
	movlw	':'-'0'
	call	a_write

	movf	capH,w				; XXX comparison
	sublw	.3
	btfss	STATUS,Z
	goto	ESR_display_2	
			; value >= 1000 => display -
	movlw	' '-'0'
	call	a_write
	movlw	'-'-'0'
	call	a_write
	movlw	'-'-'0'
	call	a_write
	movlw	' '-'0'
	call	a_write
	movlw	' '-'0'
	call	a_write
	movlw	' '-'0'
	call	a_write
	return

ESR_display_2
	call	bin2bcd
	movf	Hund,w
	btfsc	STATUS,Z
	movlw	' '-'0'
	call	a_write
	movf	Tens,w
	call	a_write
	movlw	'.'-'0'
	call	a_write
	movf	Ones,w
	call	a_write
	movlw	' '-'0'
	call	a_write
	return


read_AD0
	movlw	B'01000001'
	goto	read_AD
read_AD1
	movlw	B'01001001'
	goto	read_AD
read_AD2
	movlw	B'01010001'
	;goto	read_AD
read_AD
	movwf	ADCON0
	call	delay_100ms
	clrf	capH
	clrf	capL
	movlw	0x08
	movwf	count
read_AD_loop
	bsf	ADCON0,GO
	btfsc	ADCON0,GO
	goto	$-1
	movlw	ADRESL
	movwf	FSR
	movf	INDF,w
	addwf	capL,f
	btfsc	STATUS,C
	incf	capH,f
	movf	ADRESH,w
	addwf	capH,f
	call	delay_10ms
	decfsz	count,f
	goto	read_AD_loop
	
	movlw	0x03
	movwf	count
read_AD_avg
	clrc
	rrf	capH,f
	rrf	capL,f
	decfsz	count,f
	goto	read_AD_avg
	return


; --------------------- zero cap routine ---------------------------
; Compute the capacitance at power on (when no cap is present). This
; value will be subtracted from measure in pF mode.
; The measure is averaged over 8 samples.
; zeroHH is used during the average loop, it is not needed afterwards.
zero_pF
	bcf	T2CON,TMR2ON		; stop PWM
	call	select_pF
	movlw	B'00000100'		; pre-scale 1:1
	movwf	T1CON
	bcf	doing_mF
	clrf	capHH
	clrf	zeroH
	clrf	zeroL
	movlw	0x08			; compute zero 8 times
	movwf	count
zero_loop
	call	zero_pF_once
	movf	CCPR2L,w
	addwf	zeroL,f
	btfsc	STATUS,C
	incf	zeroH,f
	movf	CCPR2H,w
	addwf	zeroH,f
	btfsc	STATUS,C
	incf	zeroHH,f
	decfsz	count,f
	goto	zero_loop
zero_avg				; and take average
	movlw	0x03
	movwf	count
zero_avg_0
	clrc
	rrf	zeroHH,f
	rrf	zeroH,f
	rrf	zeroL,f
	decfsz	count,f
	goto	zero_avg_0
	return

zero_pF_once
	call	reset_trig_555
zpo_loop
	btfsc	captured		; captured ?
	goto	zpo_done		; yes, process value
	btfss	overflow		; overflow of timer1 ?
	goto	zpo_loop		; no, loop
	goto	display_error_zero_pf
zpo_done
	bcf	T1CON,TMR1ON		; stop timer !
	return



; divide cap by 16
	movlw	0x04
	goto	div2_0
; divide cap by 8
div_cap_8
	movlw	0x03
	goto	div2_0
; divide cap by 4
div_cap_4
	movlw	0x02
	goto	div2_0
; divide cap by 2
div_cap_2
	movlw	0x01
div2_0
	movwf	temp_1
div2_1
	clrc
	rrf	capHH,f
	rrf	capH,f
	rrf	capL,f
	decfsz	temp_1,f
	goto	div2_1
	return




; ***********************
; * measure capacitance *
; ***********************

capacitance_mode
	call	LCDClear
	movlw	.16
	movwf	ptr_pos
	call	stan_char_1	
	bcf	T2CON,TMR2ON		; stop PWM
count_pF
	clrf	type
	call	select_pF
	movlw	B'00000100'
	movwf	T1CON
	bcf	doing_mF
	clrf	capHH
	clrf	capH
	clrf	capL
	movlw	0x04			; do 4 measures and average
	movwf	count			
pF_loop
	call	reset_trig_555
pF_loop2
	btfsc	captured		; captured ?
	goto	pF_done			; yes, process value
	btfsc	overflow		; overflow of timer1 ?
	goto	count_nF		; yes, try nF scale
	goto	pF_loop2		; no, loop
pF_done
	bcf	T1CON,TMR1ON		; stop timer !
	call	add_to_cap
	call	delay_10ms		; wait 10ms before next measure
	decfsz	count,f
	goto	pF_loop
	call	div_cap_4

	movf	zeroL,w			; subtract "zero" measured capacitance
	subwf	capL,f	
	movf	zeroH,w
	btfss	STATUS,C
	incfsz	zeroH,w
	subwf	capH,f
	btfss	STATUS,C		; if result < 0, show 0
	goto	pF_zero_display
	movf	capH,f			; if value < 1pF, show 0 too
	btfss	STATUS,Z
	goto	pF_display
	movf	capL,w
	sublw	0x08
	btfss	STATUS,C
	goto	pF_display
pF_zero_display
	clrf	capHH			
	clrf	capH
	clrf	capL
pF_display
	call	display
	goto	cap_done

count_nF
	incf	type,f
	call	select_nF
	movlw	B'00000100'		; pre-scale 1:1
	movwf	T1CON
	clrf	capHH
	clrf	capH
	clrf	capL
	movlw	0x04
	movwf	count
nF_loop
	call	reset_trig_555
nF_loop2
	btfsc	captured		; captured ?
	goto	nF_done			; yes, process value
	btfsc	overflow		; overflow of timer1 ?
	goto	count_uF		; yes, try nF scale
	goto	nF_loop2		; no, loop
nF_done
	bcf	T1CON,TMR1ON
	call	add_to_cap
	call	delay_10ms
	decfsz	count,f
	goto	nF_loop
	call	div_cap_4
	call	display
	goto	cap_done

count_uF
	incf	type,f
;	btfss	fromM			; give user feedback unless
;	call	update_type		; we just displayed mF
	call	select_uF
	movlw	B'00110100'		; pre-scale 1:8
	movwf	T1CON
	clrf	capHH
	clrf	capH
	clrf	capL
	movlw	0x02			; 2 measures for uF
	movwf	count
uF_loop
	call	reset_trig_555_long
uF_loop2
	btfsc	captured		; captured ?
	goto	uF_done			; yes, process value
	btfsc	overflow		; overflow of timer1 ?
	goto	count_mF		; yes, try nF scale
	goto	uF_loop2		; no, loop
uF_done
	bcf	T1CON,TMR1ON
	call	add_to_cap
	call	delay_10ms
	decfsz	count,f
	goto	uF_loop
					; divide by 2 for average
	call	div_cap_4		; uF = divide by 16 so div by 2 first
	call	display			; since display does the div by 8
	goto	cap_done

count_mF
	bsf	doing_mF
	incf	type,f
;	call	update_type
	clrf	capL
	clrf	capH
	clrf	capHH
	call	reset_trig_555_long	; XXX very long ?
mF_loop
	btfsc	captured		; captured ?
	goto	mF_done			; yes, process value
	btfsc	overflow		; overflow ?
	goto	range_exceeded
	goto	mF_loop			; no, loop
mF_done
	bcf	T1CON,TMR1ON
	call	add_to_cap

	movlw	0x07			; 0x03E8 = 1000   0x7D0=2000
	movwf	divH
	movlw	0xD0
	movwf	divL
	call	div24_16		; divide cap by 1000
	
	call	display
	goto	cap_done

range_exceeded
	goto	display_range_exceeded

cap_done
	call	delay_100ms

	call	find_mode
	btfsc	STATUS,C
	goto	ESR_mode
	goto	count_pF


add_to_cap
	movf	CCPR2L,w
	addwf	capL,f
	btfsc	STATUS,C
	incf	capH,f
	movf	CCPR2H,w
	addwf	capH,f
	btfsc	STATUS,C
	incf	capHH,f
	return

save_cap
	movf	capL,w
	movwf	cap2L
	movf	capH,w
	movwf	cap2H
	movf	capHH,w
	movwf	cap2HH
	return

restore_cap
	movf	cap2L,w
	movwf	capL
	movf	cap2H,w
	movwf	capH
	movf	cap2HH,w
	movwf	capHH
	return


;------------------ display Ex --------------------------------
display_range_exceeded
	movlw	.56
	goto	display_error

display_error_zero_pf
	movlw	.72
display_error
	movwf	ptr_pos
	call	stan_char_1
	call	stan_char_2
endless
	nop
	goto	endless




;------------------ display capacitance --------------------------------
;
; value = capHH:capH:capL     unit = type
;
; value must be 8 times bigger. the value is first divided by 8. If the
; upper digit is 0, then value is multiplied by 10 then divided by 8, in
; order to get an extra digit, so that no resolution is lost.
display
	call	save_cap
	call	div_cap_8		; divide value by 8
	call	bin2bcd
	movf	Thou,f
	btfsc	STATUS,Z		; if Thousands == 0
	goto	display2		; then display from Hundreds
	incf	type,f			; else change scale and add dot. XXX mF
	goto	display_capa_div10
display2
	bcf	dot2
	movf	Hund,f			; if Hundreds != 0
	btfss	STATUS,Z		; then display (no decimal point)
	goto	display_capa
	call	restore_cap		; restore full-definition cap
	call	mul_cap_10		; multiply cap by 10
	call	div_cap_8		; divide by 8
	call	bin2bcd
	bsf	dot2			; add decimal dot

; set view with Hund:Tens:Ones:Type
display_capa
	call	LCDLine_2
	movlw	' '-'0'
	call	a_write
	bcf	suppress
	movf	Hund,w
	btfss	STATUS,Z
	goto	display_capa_1
	bsf	suppress
	movlw	' '-'0'
display_capa_1
	call	a_write

	movf	Tens,w
	btfss	suppress
	goto	display_capa_2
	btfsc	STATUS,Z
	movlw	' '-'0'
display_capa_2
	call	a_write
	btfss	dot2
	goto	display_capa_3
	movlw	'.'-'0'
	call	a_write
display_capa_3
	movf	Ones,w
	call	a_write
	goto	display_type

; set view with Thou:DOT:Hund:Tens:type
display_capa_div10
	call	LCDLine_2
	movf	Thou,w
	call	a_write
	movlw	'.'-'0'
	call	a_write
	movf	Hund,w
	call	a_write
	movf	Tens,w
	call	a_write
	goto	display_type

display_type
	movlw	' '-'0'
	call	a_write
	movf	type,w
	call	convert_type
	call	a_write
	movlw	'F'-'0'
	call	a_write
	movlw	' '-'0'
	call	a_write
	return

convert_type
	TABLE_JUMP
	retlw	'p'-'0'
	retlw	'n'-'0'
	retlw	0xE4-'0'
	retlw	'm'-'0'

	

	



mul_cap_10				; multiply cap by 10 (16 bits)
	movf	capL,w
	movwf	temp_1
	movf	capH,w
	movwf	temp_2
	movlw	.3
	movwf	temp_3
mul_1					; by 8 first
	clrc				
	rlf	capL,f
	rlf	capH,f
	decfsz	temp_3,f
	goto	mul_1
	movlw	.2
	movwf	temp_3
mul_2					; then add initial value twice
	movf	temp_1,w
	addwf	capL,f
	btfsc	STATUS,C
	incf	capH,f
	movf	temp_2,w
	addwf	capH,f
	decfsz	temp_3,f
	goto	mul_2
	return


;------------------ 555 control --------------------------------
; reset and trigger 555
reset_trig_555
	call	reset_555	; reset 555
	bsf	TRIG555	
	call	delay_100ms
	goto	trig

reset_trig_555_long
	call	reset_555
	bsf	TRIG555
	call	delay_1s
	goto	trig

reset_trig_555_very_long
	call	reset_555
	bsf	TRIG555
	call	delay_10s
	goto	trig

trig
	clrf	TMR1H		; reset TMR1
	clrf	TMR1L
	bcf	overflow	; reset overflow flag
	bcf	captured	; reset captured flag
	DISABLE_IRQ
	bcf	TRIG555		; trigger !
	bsf	T1CON,TMR1ON	; start timer !
	ENABLE_IRQ
	movlw	0x10		; little delay for pulse on trig line
	movwf	temp_1
rt_555	decfsz	temp_1,f
	goto	rt_555
	bsf	TRIG555
	return


; reset 555
reset_555
	bsf	RESET555
	bcf	RESET555
	call	delay_1ms
	bsf	RESET555
	return


; select range
select_pF
	bsf	SEL_UF		; deselect uF
	movlw	TRISC		; deselect nF (make input pin)
	movwf	FSR
	bsf	INDF,SEL_NF_TRIS
	return

select_nF
	bsf	SEL_UF		; deselect uF
	movlw	TRISC		; select nF (make output pin high)
	movwf	FSR
	bcf	INDF,SEL_NF_TRIS
	bsf	SEL_NF
	return

select_uF
	bcf	SEL_UF		; select uF
	movlw	TRISC		; deselect nF (make input pin)
	movwf	FSR
	bsf	INDF,SEL_NF_TRIS
	return




; *************
; * UTIL CODE *
; *************


;----------------- convert 16 bits to decimal -----------------------
; capH:capL -> Tenk:Thou:Hund:Tens:Ones
bin2bcd
	swapf	capH,w		;w  = A2*16+A3
        andlw   0x0F    	;w  = A3
        addlw   0xF0		;w  = A3-16
        movwf   Thou		;B3 = A3-16
        addwf   Thou,f		;B3 = 2*(A3-16) = 2A3 - 32
        addlw   .226		;w  = A3-16 - 30 = A3-46
        movwf   Hund		;B2 = A3-46
        addlw   .50		;w  = A3-46 + 50 = A3+4
        movwf   Ones		;B0 = A3+4

        movf    capH,w		;w  = A3*16+A2
        andlw   0x0F		;w  = A2
        addwf   Hund,f		;B2 = A3-46 + A2 = A3+A2-46
        addwf   Hund,f		;B2 = A3+A2-46  + A2 = A3+2A2-46
        addwf   Ones,f		;B0 = A3+4 + A2 = A3+A2+4
        addlw   .233		;w  = A2 - 23
        movwf   Tens		;B1 = A2-23
        addwf   Tens,f		;B1 = 2*(A2-23)
        addwf   Tens,f		;B1 = 3*(A2-23) = 3A2-69 (Doh! thanks NG)

        swapf   capL,w		;w  = A0*16+A1
        andlw   0x0F		;w  = A1
        addwf   Tens,f		;B1 = 3A2-69 + A1 = 3A2+A1-69 range -69...-9
        addwf   Ones,f		;B0 = A3+A2+4 + A1 = A3+A2+A1+4 and Carry = 0 (thanks NG)

        rlf     Tens,f		;B1 = 2*(3A2+A1-69) + C = 6A2+2A1-138 and Carry is now 1 as tens register had to be negitive
        rlf     Ones,f		;B0 = 2*(A3+A2+A1+4) + C = 2A3+2A2+2A1+9 (+9 not +8 due to the carry from prev line, Thanks NG)
        comf    Ones,f		;B0 = ~(2A3+2A2+2A1+9) = -2A3-2A2-2A1-10 (ones complement plus 1 is twos complement. Thanks SD)

        rlf     Ones,f		;B0 = 2*(-2A3-2A2-2A1-10) = -4A3-4A2-4A1-20

        movf    capL,w		;w  = A1*16+A0
        andlw   0x0F		;w  = A0
        addwf   Ones,f		;B0 = -4A3-4A2-4A1-20 + A0 = A0-4(A3+A2+A1)-20 range -215...-5 Carry=0
        rlf     Thou,f		;B3 = 2*(2A3 - 32) = 4A3 - 64

        movlw   0x07		;w  = 7
        movwf   TenK		;B4 = 7

				;B0 = A0-4(A3+A2+A1)-20	;-5...-200
				;B1 = 6A2+2A1-138	;-18...-138
				;B2 = A3+2A2-46		;-1...-46
				;B3 = 4A3-64		;-4...-64
				;B4 = 7			;7
				; At this point, the original number is
				; equal to TenK*10000+Thou*1000+Hund*100+Tens*10+Ones 
				; if those entities are regarded as two's compliment 
				; binary.  To be precise, all of them are negative 
				; except TenK.  Now the number needs to be normal- 
				; ized, but this can all be done with simple byte 
				; arithmetic.

        movlw   .10		;w  = 10
Lb1:				;do
        addwf   Ones,f		; B0 += 10
        decf    Tens,f		; B1 -= 1
        btfss   3,0
				;skip no carry
        goto   Lb1		; while B0 < 0
				;jmp carry
Lb2:				;do
        addwf   Tens,f		; B1 += 10
        decf    Hund,f		; B2 -= 1
        btfss   3,0
        goto   Lb2		; while B1 < 0
Lb3:				;do
        addwf   Hund,f		; B2 += 10
        decf    Thou,f		; B3 -= 1
        btfss   3,0
        goto   Lb3		; while B2 < 0
Lb4:				;do
        addwf   Thou,f		; B3 += 10
        decf    TenK,f		; B4 -= 1
        btfss   3,0
        goto   Lb4		; while B3 < 0

        retlw   0


;---------------- divide 24 bits by 16 bits ------------------
; divide capHH:capH:capL by divH:divL
; uses temp_1 temp_2 temp_3
div24_16
	clrf	temp_3
	clrf	temp_1
	movlw	.24
	movwf	temp_2
div24_1
	rlf	capL,f
	rlf	capH,f
	rlf	capHH,f

	rlf	temp_1,f
	rlf	temp_3,f

	rlf	temp_2,f

	movf	divL,w
	subwf	temp_1,f
	movf	divH,w
	btfss	STATUS,C
	incfsz	divH,w
	subwf	temp_3,w

	skpnc
	bsf	temp_2,0

	btfsc	temp_2,0
	goto	div24_2

	movf	divL,w
	addwf	temp_1,f
	movf	temp_3,w
div24_2
	movwf	temp_3
	clrc
	rrf	temp_2,f
	decfsz	temp_2,f
	goto	div24_1

	rlf	capL,f
	rlf	capH,f
	rlf	capHH,f

	return


; ***************
; * EEPROM CODE *
; ***************

; input: address     output: data in W
eeprom_read_byte
	BANK3
erb_1
	btfss	EECON1,WR
	goto	erb_2
	clrwdt
	goto	erb_1
erb_2
	BANK2
	movf	address,w
	movwf	EEADR
	bsf	STATUS,RP0	; bank 3
	bcf	EECON1,EEPGD
	bsf	EECON1,RD
	bcf	STATUS,RP0	; bank 2
	movf	EEDATA,w
	BANK0
	return

; input: address, data in W    output:none
eeprom_write_byte
	movwf	value
	BANK3
ewb_1
	btfss	EECON1,WR
	goto	ewb_2
	clrwdt
	goto	ewb_1
ewb_2
	bcf	STATUS,RP0	; bank 2
	movf	address,w
	movwf	EEADR
	movf	value,w
	movwf	EEDATA
	bsf	STATUS,RP0	; bank 3
	bcf	EECON1,EEPGD
	bsf	EECON1,WREN
	bcf	INTCON,GIE
	movlw	0x55
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	bsf	INTCON,GIE
	bcf	EECON1,WREN
	BANK0
	return



;---------------- clear ram ----------------------------------
clear_ram
	movlw	0x20
	movwf	FSR
clear_bank0
	clrf	INDF
	incf	FSR,f
	movlw	0x7F
	subwf	FSR,w
	bnz	clear_bank0
	movlw	0xA0
	movwf	FSR
clear_bank1
	clrf	INDF
	incf	FSR,f
	movlw	0xEF
	subwf	FSR,w
	bnz	clear_bank1
	BANK2
	movlw	0x10
	movwf	FSR
clear_bank2
	clrf	INDF
	incf	FSR,f
	movlw	0x70
	subwf	FSR,w
	bnz	clear_bank2
	movlw	0x90
	movwf	FSR
clear_bank3
	clrf	INDF
	incf	FSR,f
	movlw	0xEF
	subwf	FSR,w
	bnz	clear_bank3
	BANK0
	return


;---------------- init ports ----------------------------------
init_ports
	clrf	INTCON		; setup interrupts
	clrf	PIR1
	clrf	PIR2
	movlw	PIE1
	movwf	FSR
	clrf	INDF
	movlw	PIE2
	movwf	FSR
	clrf	INDF
	movlw	PIE1
	movwf	FSR
	bsf	INDF,TMR1IE	; enable TMR1 interrupt
	bsf	INTCON,PEIE
	movlw	PIE2
	movwf	FSR
	bsf	INDF,CCP2IE	; enable CCP2 interrupt

	movlw	ADCON0
	movwf	FSR
	movlw	B'01000001'	; FOsc/8, ADON
	movwf	INDF

	movlw	ADCON1
	movwf	FSR
	movlw	B'10000100'	; right-justified, RA0,RA1,RA3 analog
	movwf	INDF

	movlw	TRISA
	movwf	FSR
	movlw	B'11111111'	; port A : all inputs
	movwf	INDF

	movlw	TRISB
	movwf	FSR
	clrf	INDF		; port B : all outputs
	
	movlw	TRISC
	movwf	FSR
	movlw	B'00000010'	; port C : all outputs except RC1 (Out555)
	movwf	INDF

	movlw	CCP1CON		; CCP1 : PWM mode
	movwf	FSR
	movlw	B'00001111'	; 2 LSB='00', PWM mode
	movwf	INDF
	movlw	PR2
	movwf	FSR
	movlw	0x09		; PWM=[PR2 + 1] * 4 * TOsc
	movwf	INDF
	movlw	0x05		; 100kHz, duty 50%
	movwf	CCPR1L
	movlw	T2CON
	movwf	FSR
	movlw	B'00000000'	; postscale 1:1, prescale 1:1, timer2 off
	movwf	INDF

	movlw	CCP2CON		; CCP2 : Capture mode
	movwf	FSR
	movlw	B'00000100'	; capture every failing edge
	movwf	INDF

	return

	END
